------------------------------------------------------------------
----------					FILTER						----------
------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity FILTER is
   port(
		RESN		: in std_logic;
		MCLK  		: in std_logic;
		--M0, M1, M2	: in std_logic;
        MDAT        : in std_logic;
        SINC3_OUT   : out std_logic_vector(15 downto 0);
        SINC3_DONE  : out std_logic;
        CNR_TEST    : out std_logic
		);
end FILTER;

architecture RTL of FILTER is 

--==================  SINC3   ====================
   component SINC3
      port(
			RESN	: in std_logic;
			MOUT	: in std_logic;
			MCLK	: in std_logic;
			CNR 	: in std_logic;	
			CN5  	: out std_logic_vector(24 downto 0));
   end component;

	signal DECIMATION 			: std_logic_vector(2 downto 0);
	signal LOAD_CNT, COUNTER 	: std_logic_vector(8 downto 0); 	-- 2^n  -- for the decimation ratio
	signal DRDY_INT_DELAY 		: std_logic; 						-- Data Ready_Interface_Delay : for a number of clock cycles.
	signal DRDY_INT 			: std_logic;						-- Data Ready_Interface 
	signal MCLKn 				: std_logic;
	signal SINC3_OUT_INT 		: std_logic_vector(24 downto 0); 		-- interface to SINC3_OUT
	signal CNT_EXP 				: std_logic; 						
	signal CNR 					: std_logic;
begin

   MCLKn <= not MCLK;

--================	calculate decimation ratio 	============
	--  DECIMATION <= M2 & M1 & M0;
		DECIMATION <= "111";
   process(DECIMATION, SINC3_OUT_INT)
   begin
      case DECIMATION is
         -- when "000" => LOAD_CNT  <= "000000100"; 				-- 4
                       -- SINC3_OUT <= SINC3_OUT_INT(15 downto 0);
         -- when "001" => LOAD_CNT  <= "000001000"; 				-- 8
                       -- SINC3_OUT <= SINC3_OUT_INT(15 downto 0);
         -- when "010" => LOAD_CNT  <= "000010000"; 				-- 16
                       -- SINC3_OUT <= SINC3_OUT_INT(15 downto 0);
         -- when "011" => LOAD_CNT  <= "000100000"; 				-- 32
                       -- SINC3_OUT <= SINC3_OUT_INT(15 downto 0);
         -- when "100" => LOAD_CNT  <= "001000000"; 				-- 64
                       -- SINC3_OUT <= SINC3_OUT_INT(17 downto 2);
         -- when "101" => LOAD_CNT  <= "010000000"; 				-- 128
                       -- SINC3_OUT <= SINC3_OUT_INT(20 downto 5);
			when "111" => LOAD_CNT  <= "100000000"; 				-- 128
				--		SINC3_OUT <= SINC3_OUT_INT(24 downto 9);	-- 256  --- only  for 256
						SINC3_OUT <= SINC3_OUT_INT(24 downto 9);				
			when others => NULL;						

      end case;
   end process;

--================	decimation ratio 	===================
   CNR_CNT : process(MCLK, RESN)
   begin
      if RESN = '0' then
	  --  COUNTER  <= "000000100";  		-- decimation ratio 4
		COUNTER  <= "100000100";  		-- decimation ratio 256
      elsif MCLK'event and MCLK = '1' then
         if COUNTER <= "000000000" then
            COUNTER <= LOAD_CNT - 1;
         else
            COUNTER <= COUNTER - 1;
         end if;
      end if;
   end process CNR_CNT;

   CNT_EXP <= '1' when COUNTER <= "0000000000000" else '0';
   
   
   
--================	Clock MCLK 	===========================
   CNR_GEN : process(MCLKn, RESN)
   begin
      if RESN = '0' then
         CNR <= '0';
      elsif MCLKn'event and MCLKn='1' then
         CNR <= not CNT_EXP;
      end if;
   end process CNR_GEN;
   
   
   
--================	Divider Clock CNR	====================
   DRDY_GEN : process(CNR, RESN)
   begin
      if RESN = '0' then
         DRDY_INT <= '0';
      elsif CNR'event and CNR = '1' then
         DRDY_INT <= not DRDY_INT;
      end if;
   end process DRDY_GEN;

   -- DRDY signal changes polarity whenever data is ready
   -- take this information to generate set_irq signal
   -- use MCLKn to sync DRDY_INT
   
   SET_IRQ_GEN : process(RESN, MCLK)
   begin
      if RESN = '0' then
         DRDY_INT_DELAY <= '0';
         SINC3_DONE     <= '0';
      elsif MCLK'event and MCLK = '1' then
         DRDY_INT_DELAY <= DRDY_INT;
         SINC3_DONE     <= DRDY_INT xor DRDY_INT_DELAY;
      end if;
   end process SET_IRQ_GEN;

   SINC3_UNIT : SINC3 port map(
								RESN => RESN, 
								MOUT => MDAT, 
								MCLK => MCLK, 
								CNR  => CNR, 
								CN5  => SINC3_OUT_INT
								);

   CNR_TEST <= CNR;


end RTL;