------------------------------------------------------------------
----------					SINC3						----------
------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity SINC3 is
   port(
		   RESN : in std_logic;    
		   MOUT : in std_logic;    
		   MCLK : in std_logic;    
		   CNR 	: in std_logic;     
		   CN5  : out std_logic_vector(24 downto 0)
		);
end SINC3;

architecture RTL of SINC3 is                -- Changed structure of filter

signal DN0 		: std_logic_vector(24 downto 0);
signal DN1		: std_logic_vector(24 downto 0);
signal DN3  	: std_logic_vector(24 downto 0);
signal DN5 		: std_logic_vector(24 downto 0);

signal CN1 		: std_logic_vector(24 downto 0);
signal CN2 		: std_logic_vector(24 downto 0);
signal CN3		: std_logic_vector(24 downto 0);
signal CN4 		: std_logic_vector(24 downto 0);

signal DELTA1 	: std_logic_vector(24 downto 0);

begin

process(MCLK, RESN)
begin
	if RESN = '0' then
		DELTA1 <= (others => '0');
	elsif MCLK'event and MCLK = '1' then
		if MOUT = '1' then
			DELTA1 <= DELTA1 + 1;
		end if;
	end if;
end process;

process(RESN, MCLK)
begin
	if RESN = '0' then
		CN1 <= (others => '0');
		CN2 <= (others => '0');
	elsif MCLK'event and MCLK = '1' then
		CN1 <= CN1 + DELTA1;
		CN2 <= CN2 + CN1;
	end if;
end process;

process(RESN, CNR)
begin
	if RESN = '0' then
		DN0 <= (others => '0');
		DN1 <= (others => '0');
		DN3 <= (others => '0');
		DN5 <= (others => '0');
	elsif CNR'event and CNR = '1' then
		DN0 <= CN2;
		DN1 <= DN0;
		DN3 <= CN3;
		DN5 <= CN4;
	end if;
end process;
CN3 <= DN0 - DN1;
CN4 <= CN3 - DN3;
CN5 <= CN4 - DN5;

end RTL;